#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include "Serial.h"
#include "Sleep.h"
#include "ADC.h"

unsigned long TakeReading;
int Threshold;

SIGNAL(SIG_OVERFLOW0)
{
   TakeReading ++;
   if (bit_is_clear(PORTB, 0)) sbi(DDRB, 0);
   else cbi(DDRB, 0);
   return;
} 


int Init()
{
   int Prescaler;

   outb(DDRB, 0x01);

   cli();
   TakeReading = 0;
   Prescaler = 5;
//   outb(TCCR0, Prescaler & 0x07);
//   outb(TCCR0, 0x05);
   sbi(TCCR0, 0);
   sbi(TCCR0, 2);
   sbi(TIFR, TOV0);
   sbi(TIMSK, TOIE0);
   outb(TCNT0, 0);

   Threshold = 20;

//   timer_enable_int(1 << TOIE0);
   return (0);
}

int OutputReading(unsigned long ADCValue)
{
   unsigned char Buffer[100];

   itoa(ADCValue, Buffer, 10);
   UART_TransmitLine("Reading: ");
   UART_TransmitLine(Buffer);
   UART_TransmitLine("\n\r");
   return(0);
}

unsigned long GetReading()
{
   unsigned long ADCValue;
   unsigned long ADCAccumulation;
   int Counter;

   ADCAccumulation = 0;
   for (Counter = 0 ; Counter < 4 ; Counter ++)
   {
      ADCValue = StartADC(3);
      ADCAccumulation += ADCValue;
   }
   ADCValue = ADCAccumulation / 4;
   return(ADCValue);
}

int ReadEeproms()
{
   int Counter;
   unsigned char Result;
   unsigned char Buffer[10];

//   eeprom_wb(uint16_t addr, uint8_t val);
   eeprom_wb(0, ' ');

   for (Counter = 0 ; Counter < 16 ; Counter ++)
   {
      itoa(Counter, Buffer, 16);
      UART_TransmitLine("Addr 0x");
      UART_TransmitLine(Buffer);
      UART_TransmitLine(": ");
      Result = eeprom_rb(Counter);
      itoa(Result, Buffer, 16);
      UART_TransmitLine(Buffer);
      UART_TransmitLine("\n\r");
   }
   return (0);
}

int main(int argc, char *argv[])
{
   unsigned long ADCValue;
   unsigned long LastValue;

   UART_Init(51);

   SetupADC(3, 1);

   Init();
   sei();

   ADCValue = StartADC(3);

   UART_clr();
//   ReadEeproms();
   UART_TransmitLine("Starting up:\n\r");
   ADCValue = GetReading();
   OutputReading(ADCValue);
   LastValue = ADCValue;
   while (1)
   {
      while (TakeReading < 5) ;
//      InputChar = UART_ReceiveByte();
      ADCValue = GetReading();

      if ((ADCValue >= (LastValue + Threshold)) ||
         (LastValue >= (ADCValue + Threshold)))
      {
         OutputReading(ADCValue);
         LastValue = ADCValue;
      }

      TakeReading = 0;
   }
}
