#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>

#define PodType POD_TYPE_LIGHTSENSOR
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/LightSensor/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"
#include "Sleep.h"
#include "ADC.h"

/* Locat POD Variables */
#define POD_VAR_THRESHOLD_ADDR 0x0040
#define POD_VAR_THRESHOLD_LEN  0x0002

int Threshold;

//SIGNAL(SIG_OVERFLOW0)
//{
//   TakeReading ++;
//   if (bit_is_clear(PORTB, 0)) sbi(DDRB, 0);
//   else cbi(DDRB, 0);
//   return;
//} 

void ReadVars()
{
   Threshold = eeprom_read_word((uint16_t *)(POD_VAR_THRESHOLD_ADDR));
}

int Init()
{
   int Prescaler;

   cli();
   Prescaler = 5;
   UART_Init(23);
   SetupADC(0, 2);
   ReadVars();

   sei();
   return (0);
}

int OutputReading(unsigned long ADCValue)
{
   unsigned char Buffer[100];

   itoa(ADCValue, Buffer, 10);
   UART_TransmitLine("Reading: ");
   UART_TransmitLine(Buffer);
   UART_TransmitLine("\n\r");
   return(0);
}

void Process_Command_GetIntensity()
{
   unsigned long CurrentIntensity;

   CurrentIntensity = ADC_GetReading();
   UART_TransmitByte(POD_RESPONSE_ACK);
   UART_TransmitWord(CurrentIntensity & 0xFFFF);
}

void Process_Command_GetThreshold()
{
   UART_TransmitByte(POD_RESPONSE_ACK);
   UART_TransmitWord(Threshold);
}

void Process_Command_SetThreshold()
{
   Threshold = UART_ReceiveWord();
   eeprom_write_byte((uint8_t *)(POD_VAR_THRESHOLD_ADDR), (Threshold >> 8));
   eeprom_write_byte((uint8_t *)(POD_VAR_THRESHOLD_ADDR + 1), (Threshold & 0xFF));
   UART_TransmitByte(POD_RESPONSE_ACK);
}

void Process_Command_IsLightOn()
{
   unsigned long CurrentIntensity;

   CurrentIntensity = ADC_GetReading();
   UART_TransmitByte(POD_RESPONSE_ACK);
   if (CurrentIntensity >= (unsigned long)(Threshold)) UART_TransmitByte(0x01);
   else UART_TransmitByte(0x00);
}

int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break;
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
      case POD_COMMAND_RESET:
/*         Process_Command_Reset(); */
         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_LIGHTSENSOR_GET_INTENSITY:
         Process_Command_GetIntensity();
         break;
      case POD_COMMAND_LIGHTSENSOR_GET_THRESHOLD:
         Process_Command_GetThreshold();
         break;
      case POD_COMMAND_LIGHTSENSOR_SET_THRESHOLD:
         Process_Command_SetThreshold();
         break;
      case POD_COMMAND_LIGHTSENSOR_IS_LIGHT_ON:
         Process_Command_IsLightOn();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}

int main(int argc, char *argv[])
{
   unsigned long ADCValue;
   unsigned int Command;

   Init();
   ADCValue = StartADC(0);

   ADCValue = ADC_GetReading();
   while (1)
   {
      if (UART_Data()) /* some data is waiting */
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      }
   }
}
