#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>

/* unsigned int PodType; */
#define PodType POD_TYPE_MOTORCONTROL
#define POD_VAR_URL "http://pel.cs.byu.edu/~sorenson/Thesis/PODs/MotorControl/"

#include "PODDefines.h"
#include "PODIncludes.h"

#include "Serial.h"
#include "Sleep.h"
#include "ADC.h"

SIGNAL(SIG_OVERFLOW2)
{
}

char SetPWM(unsigned char PWMVal)
{
   outb(OCR2, PWMVal);
   return(0);
}

void SetDirection(char Direction)
{
//   cbi(PORTB, PB1);
//   cbi(PORTB, PB2);

   if (Direction == 3)
   {
      cbi(PORTB, PB1);
      cbi(PORTB, PB2);
   }
   else if (Direction == 1)
   {
      cbi(PORTB, PB1);
      sbi(PORTB, PB2);
   }
   else
   {
      cbi(PORTB, PB2);
      sbi(PORTB, PB1);
   }
}

char SetupPWM()
{
   outb(TCCR2, (1 << WGM20) | (1 << CS20) | (1 << COM20) | (1 << COM21));
   sbi(DDRB, PB1);
   sbi(DDRB, PB2);
   sbi(DDRB, PB3); /* OC2 */
   return(0);
}

char Init()
{
   cli();
   UART_Init(23);
   SetupPWM();
   sei();
   return(0);
}

void Process_Command_Stop()
{
   SetPWM(255);
   SetDirection(3); /* turn off enable */
   UART_TransmitByte(POD_RESPONSE_ACK);
}

void Process_Command_Set_Direction()
{
   unsigned char Direction;
   Direction = UART_ReceiveByte();
   SetPWM(255); /* off */
   SetDirection(Direction);
   UART_TransmitByte(POD_RESPONSE_ACK);
}

void Process_Command_Set_Speed()
{
   unsigned char Speed;
   Speed = UART_ReceiveByte();
   SetPWM(255 - Speed);
//   SetDirection(Direction);
   UART_TransmitByte(POD_RESPONSE_ACK);
}


int Process_Command(unsigned int Command)
{
   switch (Command)
   {
      case POD_COMMAND_PING:
         Process_Command_Ping();
         break;
      case POD_COMMAND_GETTYPE:
         Process_Command_GetType();
         break;
      case POD_COMMAND_GETINFO:
         Process_Command_GetInfo();
         break;
      case POD_COMMAND_RESET:
/*         Process_Command_Reset(); */
         break;
      case POD_COMMAND_GETNAME:
         Process_Command_GetName();
         break;
      case POD_COMMAND_SETNAME:
         Process_Command_SetName();
         break;
      case POD_COMMAND_MOTORCONTROL_STOP:
         Process_Command_Stop();
         break;
      case POD_COMMAND_MOTORCONTROL_SET_DIR:
         Process_Command_Set_Direction();
         break;
      case POD_COMMAND_MOTORCONTROL_SET_SPEED:
         Process_Command_Set_Speed();
         break;
      default:
         Send_NACK();
         break;
   }
   return(0);
}


int main(int argc, char *argv[])
{
   unsigned int Command;


   Init();

   SetDirection(1);
   SetPWM(255); /* off */
   while (1)
   {
      if (UART_Data()) /* some data is waiting */
      {
         Command = UART_ReceiveWord();
         Process_Command(Command);
      } /* if UART_Data */
   } /* end while */
   return(0);
}
