#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/twi.h>
#include "Serial.h"

typedef unsigned char  u08;
typedef          char  s08;
typedef unsigned short u16;
typedef          short s16;
typedef u08            bool;

#define SONAR_ADDRESS 0xE0
#define READ 1
#define WRITE 0

#define SONAR_COMMAND_REGISTER 0
//#define SONAR_COMMAND 80 //result in inches
#define SONAR_COMMAND 0x51 //result in cm
#define SONAR_MAX_ADDRESS 35 //last byte available (36 bytes 0:35)
//#define SYSCLK 14745600UL
//#define SYSCLK   3686400UL
#define SYSCLK   8000000UL


void sonar_init(void)
{
   outb(PORTC, 0x30);
//   outb(DDRC, 0x30); // SDA, SCL are outputs
//   outb(PORTC, inp(PORTC) & ~(0x03));
   outb(TWBR, (SYSCLK / 100000UL - 16) / 2);
//   outb(TWBR, 21); // clock - 21 is 100khz at @6mhz
   outb(TWSR, 0);
   outb(TWCR, _BV(TWEA) | _BV(TWEN)); // ack, en, and clear irq
   outb(TWAR, 0x01);
}

s08 sonar_start(void)
{
    loop_until_bit_is_clear(TWCR, TWSTO);  //check no transmission in progress
//UART_TransmitByte('a');
 
    outb(TWCR, _BV(TWSTA) | _BV(TWEN) | _BV(TWINT)); //send start condition
//UART_TransmitByte('b');
    while(!(inp(TWCR) & _BV(TWINT))) {;} //wait for interrupt flag to get set
//UART_TransmitByte('c');
    if((inp(TWSR) & 0xF8) != TW_START)
    {
        outb(TWCR, _BV(TWINT)|_BV(TWSTO)|_BV(TWEN));
        return -1;  //error occured - should have been start
    } 
//UART_TransmitByte('d');

    TWDR = SONAR_ADDRESS | WRITE;  //send sonar address
    TWCR = _BV(TWINT) | _BV(TWEN); //clear interrupt to start transmission
    while(!(TWCR & _BV(TWINT))){;} //wait for interrupt flag to get set
    if(TWSR != TW_MT_SLA_ACK) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return -2;
    }

    TWDR = SONAR_COMMAND_REGISTER;  //send the address on the sonar to write to
    TWCR = _BV(TWINT) | _BV(TWEN); //clear IRQ, continue transmission
    while(!(TWCR & _BV(TWINT))){;} //wait for interrupt flag to get set
    if(TWSR != TW_MT_DATA_ACK) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return -3;
    }

    TWDR = SONAR_COMMAND;          //send the sonar "start ranging" command
    TWCR = _BV(TWINT) | _BV(TWEN);
    while(!(TWCR & _BV(TWINT))){;} //wait for interrupt flag to get set
    if(TWSR != TW_MT_DATA_ACK){
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return -4;
    }
    
    TWCR = _BV(TWINT) | _BV(TWSTO) | _BV(TWEN); //send stop condition
    return 0;
}

u08 sonar_get_data(u08* data) 
{
   loop_until_bit_is_clear(TWCR, TWSTO); //wait for last transmission to end
        
   TWCR = _BV(TWINT)|_BV(TWSTA)|_BV(TWEN); //send start condition
   loop_until_bit_is_set(TWCR, TWINT);
   if(TWSR != TW_START) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
   return -1;
   }

    TWDR = SONAR_ADDRESS | WRITE;         //send sonar address
    TWCR = _BV(TWINT)|_BV(TWEN);
    loop_until_bit_is_set(TWCR, TWINT);
    if(TWSR != TW_MT_SLA_ACK) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return 1;            //expected error when sonar is busy
    }

    TWDR = 0;    //send address to start reading at (register 0)
    TWCR = _BV(TWINT)|_BV(TWEN);
    loop_until_bit_is_set(TWCR, TWINT);
    if(TWSR != TW_MT_DATA_ACK) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return 1;                  //expected error when sonar is busy
    }

    TWCR = _BV(TWINT)|_BV(TWSTA)|_BV(TWEN);  //send repeated start
    loop_until_bit_is_set(TWCR, TWINT);
    if(TWSR != TW_REP_START) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN);
        return -4;
    }

    TWDR = SONAR_ADDRESS | READ;            //send sonar address for read
    TWCR = _BV(TWINT)|_BV(TWEA)|_BV(TWEN);
    loop_until_bit_is_set(TWCR, TWINT);
    if(TWSR != TW_MR_SLA_ACK) {
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return -5;
    }

    u08 i;
    for(i=0; i<SONAR_MAX_ADDRESS; i++) {
        TWCR = _BV(TWINT)|_BV(TWEA)|_BV(TWEN);
        loop_until_bit_is_set(TWCR, TWINT);
        if(TWSR != TW_MR_DATA_ACK) {
            TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
            return -6;
        }
        *data = TWDR;
        data++;
    }
    TWCR = _BV(TWINT)|_BV(TWEN);   //TWEA not set - to indicate this is last byte
    loop_until_bit_is_set(TWCR, TWINT);
    if(TWSR != TW_MR_DATA_NACK) {   //get NACK since we didn't set TWEA
        TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN); 
        return -7;
    }
    *data = TWDR;  //get last byte
    
    TWCR = _BV(TWINT)|_BV(TWSTO)|_BV(TWEN);  //send stop condition
    
    return 0;
}

void convert_data(u16* ranges, u08* light_sensor, u08* data) 
{
    data++;
    *light_sensor = *data; //1st element is light sensor
    data++;         //first range data byte (high byte)
    u08 i;
    for(i=0; i<17; i++) {
        *ranges = (*data)<<8 | *(data+1);
        data+=2;
        ranges++;
    }
}

s08 sonar_data(u16* ranges, u08* light_sensor) 
{
    u08 data[36];
    s08 error;
    do {
        error = sonar_get_data(data);
    } while(error==1);
    if(error == 0) {
        convert_data(ranges, light_sensor, data);
        return 0;
    } else {
        return -1;
    }    
}

u16 sonar_get_closest(u16* ranges)
{ 
    while(*ranges <= 6 && *ranges!=0) {
        ranges++;
    }
    return *ranges;
}

u16 sonar_closest(void)
{
    u16 ranges[17];
    u08 temp;
    s08 error;
    error = sonar_data(ranges, &temp);
    if(error == 0) {
        return sonar_get_closest(ranges);
    } else {
        return 0;
    }
    
}

void delay(u16 delay_time)
{
   do
   {
      u08 i=0;
      do
      {
         asm volatile ("nop");
         asm volatile ("nop");
         asm volatile ("nop");
         asm volatile ("nop");
      } while(--i);
   } while(--delay_time);
}

void Init()
{
//   outb(DDRA, 0);
   outb(DDRB, 0);
   outb(DDRC, 0);
   outb(DDRD, 0);

   delay(0x2000);

   UART_Init(51);
   sonar_init();
   sei();
}

int main(void)
{
   u16 ranges[17];
   u08 light_sensor;
   char Output[64];
   u16 closest;
   s08 error1, error2;
   u08 i;


   Init();
   UART_clr();
   UART_TransmitLine("Starting:\n\r");

   while (1)
   {
//      UART_TransmitByte('0');
      error1 = sonar_start();
//      UART_TransmitByte('1');
      error2 = sonar_data(ranges, &light_sensor);
//      UART_TransmitLine("2");
      closest = sonar_closest();
//      UART_TransmitLine("3");

UART_clr();
      if (error1 == 0 && error2 == 0)
      {
         for (i = 0 ; i < 17 ; i ++)
         {
            if (i%8 == 0) UART_TransmitLine("\n\r");
            sprintf(Output, " %i: %i ", i, ranges[i]);
            UART_TransmitLine(Output);
         }
         UART_TransmitLine("\n\r");
      }
      sprintf(Output, "closest: %i\tlight: %i\r\nerror1: %i error2: %i\n\r",
            closest, light_sensor, error1, error2);
      UART_TransmitLine(Output);
      delay(0x2000);
   }
}

