//#define DEBUG 1

#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>
#include <avr/twi.h>
#include "Serial.h"
#include "TWI.h"
#include "Delay.h"

#define SONAR_ADDRESS 0xE0

#define SONAR_COMMAND_REGISTER 0
//#define SONAR_COMMAND 80 //result in inches
#define SONAR_COMMAND 0x51 //result in cm
#define SONAR_MAX_ADDRESS 35 //last byte available (36 bytes 0:35)
//#define SYSCLK 14745600UL
//#define SYSCLK   3686400UL
#define SYSCLK   8000000UL


char SonarStart(void)
{
   unsigned char Data[5];
   char Result;

   Data[0] = 0x51;
   
   Result = TWI_SendCommand(SONAR_ADDRESS, SONAR_COMMAND_REGISTER, Data, 1);
   if (Result) return Result;
   delay(0x0200);

   return 0;
}

char GetData(unsigned char *Data)
{
   char Result;

   Result = TWI_GetData(SONAR_ADDRESS, 0, Data, 0x24);
   if (Result) return -1;
   return 0;
}

void Init()
{
//   outb(DDRA, 0);
   outb(DDRB, 0);
   outb(DDRC, 0);
   outb(DDRD, 0);

   delay(0x0100);

   UART_Init(51);
   TWI_Init(SYSCLK);
   sei();
}

int main(void)
{
   unsigned char RawData[36];
   char Output[64];
   unsigned int Closest;
   char error1, error2;
   unsigned char i;
   unsigned char Light;


   Init();
   UART_clr();
   UART_TransmitLine("Starting:\n\r");

   while (1)
   {
UART_clr();
UART_TransmitLine("Start...");
      error1 = SonarStart();
UART_TransmitLine("Read...");
      error2 = GetData(RawData);

      Light = RawData[1];
      Closest = (RawData[2] << 8) | RawData[3];
      if (error1 == 0 && error2 == 0)
      {
         for (i = 0 ; i < 17 ; i ++)
         {
            if (i%8 == 0) UART_TransmitLine("\n\r");
            sprintf(Output, " %i: %i ", i, (RawData[(2 * i) + 2] << 8)+ RawData[(2 * i) + 3]);
            UART_TransmitLine(Output);
         }
         UART_TransmitLine("\n\r");
      }
      sprintf(Output, "Closest: %i\tLight: %i\r\nerror1: %i error2: %i\n\r",
            Closest, Light, error1, error2);
      UART_TransmitLine(Output);
      delay(0x2000);
   }
}

