#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <inttypes.h>
#include <avr/eeprom.h>
#include <string.h>
#include "Serial.h"
#include "Sleep.h"
#include "ADC.h"

//char CompileDate __attribute__ ((section (".eeprom"))) = __DATE__;

/* POD Variables */

/* WarningType - Warning Type
 * -1 - Low temperature warning
 * 0  - No warning set (default)
 * 1  - High temperature warning
 * */
/* char WarningType __attribute__ ((section (".eeprom"))); */
char WarningType;

/* WarnTemperature - Temperature of warning (in Fahrenheit tenths - 50F will show as 500 */
/* int WarnTemperature __attribute__ ((section (".eeprom"))); */
int WarnTemperature;

/* DisplayUnits - Units to display temperature in
 * 0 - Fahrenheit
 * 1 - Celsius
 * */
/* unsigned char DisplayUnits __attribute__ ((section (".eeprom"))); */
unsigned char DisplayUnits;

/* WarningState - whether the system is already in a warning state - internally used */
char WarningState;

/* CommandWaiting - Whether there's a command waiting in the UART Buffer */
//char CommandWaiting;

int Init()
{
   int Prescaler;

   cli();
   Prescaler = 5;
   UART_Init(23);
   SetupADC(0, 1);

   WarningState = 0;
   WarnTemperature = 70;
   WarningType = 0;
   DisplayUnits = 0;

   sei();
   return(0);
}

unsigned long GetReading()
{
   unsigned long ADCValue;
   unsigned long ADCAccumulation;
   int Counter;

   ADCAccumulation = 0;
   for (Counter = 0 ; Counter < 16 ; Counter ++)
   {
      ADCValue = StartADC(0);
      ADCAccumulation += ADCValue;
   }
   ADCValue = ADCAccumulation / 16;
   return(ADCValue);
}

int SendAlert(int AlertType)
{
   if (AlertType == 1) UART_TransmitLine("pod.alert.temperature.high");
   else if (AlertType == -1) UART_TransmitLine("pod.alert.temperature.low");
   return(0);
}

/* Assumes temperature coming in as F */
unsigned long TemperatureConvert(unsigned long InputTemp, char Unit)
{
   unsigned long ReturnValue;
   if (Unit == 0) return(InputTemp);
   if (Unit == 1)
   {
      ReturnValue = ((InputTemp - 3200) * 5) / 9;
   }
   return(ReturnValue);
}

int DoCommand(unsigned char *CommandBuffer)
{
/*   unsigned char CommandBuffer[64]; */
   unsigned long ADCValue;
   unsigned long TemperatureInt;
   unsigned char TempBuffer[64];
   
/*   UART_ReceiveLine(CommandBuffer); */

   if (!strncmp(CommandBuffer, "pod.get.", 8))
   {
      if (!strncmp(CommandBuffer + 8, "temp", 5))
      {
         ADCValue = GetReading();
         TemperatureInt = TemperatureConvert((ADCValue * 25600) / 1024, DisplayUnits);
         
         itoa(TemperatureInt / 100, TempBuffer, 10);
         UART_TransmitLine("pod.response=");
         UART_TransmitLine(TempBuffer);
         UART_TransmitByte('.');
         itoa(TemperatureInt % 100, TempBuffer, 10);
         UART_TransmitLine(TempBuffer);
         UART_TransmitLine("\n\r");
      }
      else if (!strncmp(CommandBuffer + 8, "units", 6))
      {
         UART_TransmitLine("pod.response=");
         if (DisplayUnits == 0) UART_TransmitLine("F\n\r");
         else if (DisplayUnits == 1) UART_TransmitLine("C\n\r");
      }
      else if (!strncmp(CommandBuffer + 8, "info", 5))
      {
         UART_TransmitLine("pod.response=Compiled on ");
         UART_TransmitLine(__DATE__);
         UART_TransmitLine(" at ");
         UART_TransmitLine(__TIME__);
         UART_TransmitLine("\n\r");
      }
   }
   else if (!strncmp(CommandBuffer, "pod.set.", 8))
   {
      if (!strncmp(CommandBuffer + 8, "units=", 6))
      {
         if (CommandBuffer[14] == 'f') DisplayUnits = 0;
         else if (CommandBuffer[14] == 'c') DisplayUnits = 1;
         UART_TransmitLine("pod.response=ok\n\r");
      }
   }
   else
   {
      if (CommandBuffer[0] != '\0')
      {
         UART_TransmitLine("Unknown command:#");
         UART_TransmitLine(CommandBuffer);
         UART_TransmitLine("#\n\r");
      }
   }
   return(0);
}

int main(int argc, char *argv[])
{
   unsigned long ADCValue;
   unsigned long LastValue;
   unsigned long TemperatureInt;
   long CurrentTemp;
   long LastTemp;
   unsigned char InputChar;
/*   unsigned char TempBuffer[100]; */
   unsigned char CommandBuffer[80];
   unsigned char CommandBufferCounter;
   unsigned char CommandReady;

   Init();

   CommandBuffer[0] = '\0';
   CommandBufferCounter = 0;
   CommandReady = 0;

   ADCValue = StartADC(0);

   UART_clr();
//   UART_TransmitLine("Starting1\n\r");
   ADCValue = GetReading();
   LastTemp = GetReading();
   while (1)
   {
      CurrentTemp = GetReading();
      if (CurrentTemp != LastTemp)
      {
         if (WarningType == 1) /* check for over-temperature */
         {
            if (CurrentTemp >= WarnTemperature)
            {
               if (!WarningState)
               {
                  SendAlert(1);
                  WarningState = 1;
               }
            } /* if over temp */
         } /* check for over temp */
         else if (WarningType == -1) /* check for low-temperature */
         {
            if (CurrentTemp <= WarnTemperature)
            {
               if (!WarningState)
               {
                  SendAlert(-1);
                  WarningState = 1;
               }
            } /* if low temp */
         } /* check for low temp */
      } /* temperature has changed */
      if (UART_Data()) /* some data is waiting */
      {
//         UART_TransmitByte('.');
         InputChar = UART_ReceiveByte();

         if ((InputChar == '\n') || (InputChar == '\r'))
         {
            CommandReady = 1;
         }
         else if (((InputChar == '\b') || (InputChar == 127)) && (CommandBufferCounter > 0))
         {
            CommandBufferCounter --;
            CommandBuffer[CommandBufferCounter] = '\0';
//            UART_TransmitLine("\b \b"); /* if we want to echo */
         }
         else
         {
            CommandBuffer[CommandBufferCounter++] = InputChar;
//            UART_TransmitByte(InputChar);
            CommandBuffer[CommandBufferCounter] = '\0';
         }
         if (CommandReady)
         {
//            UART_TransmitLine("Got a command?\n\r");
            DoCommand(CommandBuffer);
            CommandBuffer[0] = '\0';
            CommandBufferCounter = 0;
            CommandReady = 0;
         }
      } /* if UART_Data */
   } /* end while */
   return(0);
}
